#include <vector>
#include "scene.h"


using namespace VgMath;

static char toChar(Scalar brightness){
    if (brightness<0.1) return ' ';
    if (brightness<0.3) return '.';
    if (brightness<0.5) return '-';
    if (brightness<0.7) return '+';
    if (brightness<0.9) return '*';
    return '#';
}


void raytracing( int res, Scalar focal ){

    Scene s;
    s.createRandom(5);

    Versor3 lightDir = Vector3( 2, 5, -5 ).normalized();

    Point3 pov (0,0,0); // because we are in VIEW space!

    for (int yPixel=0; yPixel<res; yPixel++ ){
        for (int xPixel=0; xPixel<res; xPixel++){
            Scalar clipX = +Scalar(2)*xPixel/res - 1;
            Scalar clipY = -Scalar(2)*yPixel/res + 1;

            Point3 pixelPos(clipX, clipY, focal );
            Ray r( pov, pixelPos );

            char pixel = '.'; // background

            Versor3 n = Versor3::up();

            if (intersect(r,s,n)) {
                Scalar diffuse = dot( n, lightDir );
                pixel = toChar( diffuse );
            }
            std::cout << pixel << ' '; // 1 pixel = two chars of space
        }
        std::cout << std::endl;
    }

}
