#pragma once
#include "point3.h"
#include "versor3.h"

namespace VgMath{

struct Sphere{
    Point3 center;
    Scalar radius;
};

struct Ray{
    Point3 start;
    Versor3 dir;

    Point3 pointAt(Scalar k) const {
        return start + k * dir;
    }

    Ray(Point3 s, Versor3 d):start(s),dir(d){ }
    Ray(Point3 s, Vector3 d):Ray(s,d.normalized()){ }
    Ray(Point3 a, Point3 b):Ray(a,b-a){ }

};

// if they intersect, n is the normal of hit point
inline bool intersect(const Ray& r, const Sphere& s, Versor3 &n){
    Scalar a = 1;
    Scalar b = dot( 2 * r.dir , r.start - s.center  );
    Scalar c = (r.start-s.center).squaredNorm() - s.radius*s.radius;

    // find k such that a k^2 + b k + c = 0
    Scalar delta = b*b - 4*a*c;
    if (delta < 0 ) return false; // no intersection
    Scalar k = (- b - std::sqrt(delta)) / (2*a);
    if (k<0) return false; // sphere behind the ray

    n = (r.pointAt(k) - s.center).normalized();

    return true;
}



}
